/* -----------------------------------------------------------------------------
 * Copyright (c) 2016 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        01. February 2016
 * $Revision:    V1.00
 *
 * Project:      Flash Programming Functions for Kinetis
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.00
 *    Initial release
 */

#include "..\FlashOS.H"        /* FlashOS Structures */
#include "SSD_Types.h"
#include "SSD_FTFx.h"
#include "SSD_FTFx_Internal.h"
#include "user_cfg.h"
#include "C90TFS_AddOn.h"
#include "C90TFS_Device.h"


#define STACK_SIZE   1024      /* Stack Size */


unsigned long base_adr;

FLASH_SSD_CONFIG flashSSDConfig;
uint8_t unsecure_key[PGM_SIZE_BYTE];


/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */
int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {
  int ret;

  /* store base address */
  base_adr = adr;

  /* initialize unsecure key */
  #if (PGM_SIZE_BYTE == 4)
    unsecure_key[0] = 0xFEU;
    unsecure_key[1] = 0x3FU;
    unsecure_key[2] = 0xFFU;
    unsecure_key[3] = 0xFFU;
  #else
    unsecure_key[0] = 0xFFU;
    unsecure_key[1] = 0xFFU;
    unsecure_key[2] = 0xFFU;
    unsecure_key[3] = 0xFFU;
    unsecure_key[4] = 0xFEU;
    unsecure_key[5] = 0x3FU;
    unsecure_key[6] = 0xFFU;
    unsecure_key[7] = 0xFFU;
  #endif

  /* set run mode (remove low power mode) */
  RunMode_Set();

  /* disable the watchdog */
  Watchdog_Disable();

  /* initialize SSD Configuration Structure */
  flashSSDConfig.ftfxRegBase     = FTFx_REG_BASE;             /* FTFx control register base */
  flashSSDConfig.PFlashBlockBase = PFLASH_BLOCK_BASE;         /* base address of PFlash block */
  flashSSDConfig.PFlashBlockSize = PFLASH_BLOCK_SIZE;         /* size of PFlash block */
  flashSSDConfig.DFlashBlockBase = DEFLASH_BLOCK_BASE;        /* base address of DFlash block */
  flashSSDConfig.DFlashBlockSize = DEFLASH_BLOCK_SIZE;        /* size of DFlash block */
  flashSSDConfig.EERAMBlockBase  = EERAM_BLOCK_BASE;          /* base address of EERAM block */
  flashSSDConfig.EEEBlockSize    = EERAM_BLOCK_SIZE;          /* size of EEE block */
  flashSSDConfig.DebugEnable     = 0;                         /* background debug mode enable bit */
  flashSSDConfig.CallBack        = NULL_CALLBACK;             /* pointer to callback function */

  ret = pFlashInit(&flashSSDConfig);
  if (ret == FTFx_OK)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

  return (0);                                                 /* Finished without Errors */
}


/*  Blank Check Block in Flash Memory
 *    Parameter:      adr:  Block Start Address
 *                    sz:   Block Size (in bytes)
 *                    pat:  Block Pattern
 *    Return Value:   0 - OK,  1 - Failed
 */

int BlankCheck (unsigned long adr, unsigned long sz, unsigned char pat) {
  int ret;

  ret = pFlashVerifySection(&flashSSDConfig, adr, sz/PRD1SEC_ALIGN_SIZE, READ_NORMAL_MARGIN, pFlashCommandSequence);
  if (ret == FTFx_OK)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}


/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */
int EraseChip (void) {
  int ret;

  ret = pFlashEraseAllBlock(&flashSSDConfig,pFlashCommandSequence);
  if (ret != FTFx_OK)
  {
    return (1);
  }

  /* Program SECURITY bit */
  ret = pFlashProgram(&flashSSDConfig, SECURITY_LOCATION, SECURITY_SIZE, (uint8_t *)unsecure_key, pFlashCommandSequence);
  if (ret == FTFx_OK)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */
int EraseSector (unsigned long adr) {
  int ret;

#if (defined D32_2KBSec) || (defined D64_2KBSec)
  ret = pFlashEraseSector(&flashSSDConfig, adr, FTFx_DSECTOR_SIZE, pFlashCommandSequence);
#else
  ret = pFlashEraseSector(&flashSSDConfig, adr, FTFx_PSECTOR_SIZE, pFlashCommandSequence);
#endif
  if (ret == FTFx_OK)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */
int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
  int ret;

  sz = (sz + (PGM_SIZE_BYTE - 1)) & ~(PGM_SIZE_BYTE - 1);     /* Align to PGM_SIZE_BYTE */

  ret = pFlashProgram(&flashSSDConfig, adr, sz, buf, pFlashCommandSequence);
  if (ret == FTFx_OK)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}
